// Версия ПО v1.1
// Изменения по сравнению с v1.0
// - переработана функция отображения данных на индикаторе MP1091

#include <Wire.h>
#include <EEPROM.h>
#include <Time.h>
#include <PinChangeInt.h>
#include <MP1092.h>
#include <MP1091.h>
#include <MP1090S.h>


const int KEY_0 = 2;
const int KEY_1 = 3;
const int KEY_2 = 4;

const int KEY_PRESSED = 0;
const int KEY_UNPRESSED = 1;

const int MP1090S_RST_PIN = 13;

const int MP1091_DIG1_PIN = 8;
const int MP1091_DIG2_PIN = 9;
const int MP1091_DIG3_PIN = 10;
const int MP1091_DIG4_PIN = 11;

const int MP1091_A_MP1092_PIN = 0;
const int MP1091_B_MP1092_PIN = 1;
const int MP1091_C_MP1092_PIN = 2;
const int MP1091_D_MP1092_PIN = 3;
const int MP1091_E_MP1092_PIN = 4;
const int MP1091_F_MP1092_PIN = 5;
const int MP1091_G_MP1092_PIN = 6;
const int MP1091_DP_MP1092_PIN = 7;
const int MP1091_PP_MP1092_PIN = 8;

const int MP1092_ADDRESS = 0x60;
const int MP1092_PIN_HIGH = 0;
const int MP1092_PIN_LOW = 1;
const int MP1092_PIN_PWM0 = 2;

const int MP410_GAIN0_MP1092_PIN = 12;
const int MP410_GAIN1_MP1092_PIN = 13;
const int MP410_PWDR_MP1092_PIN = 14;
const int MP410_PWDL_MP1092_PIN = 15;

const int MP1093_VOL_A = 14;
const int MP1093_VOL_B = 15;
const int MP1093_TUNE_A = 16;
const int MP1093_TUNE_B = 17;
const int MP1093_CounterClockWiseRot = 0;
const int MP1093_ClockWiseRot = 1;
const int MP1093_Unchanged = 2;

const int MIN_VOLUME = 0;
const int MAX_VOLUME = 15;

const int InitVolume = 8;
const long InitStation = 96800;

const int STATION_0_EEPROM_ADDRESS = 0x00;
const int STATION_1_EEPROM_ADDRESS = STATION_0_EEPROM_ADDRESS+4;
const int STATION_2_EEPROM_ADDRESS = STATION_1_EEPROM_ADDRESS+4;

volatile byte VOLUME_FLAG = MP1093_Unchanged;
volatile byte TUNE_FLAG = MP1093_Unchanged;

volatile byte KEY_0_FLAG = KEY_UNPRESSED;
volatile byte KEY_1_FLAG = KEY_UNPRESSED;
volatile byte KEY_2_FLAG = KEY_UNPRESSED;

volatile long STATION_0;
volatile long STATION_1;
volatile long STATION_2;

volatile boolean STATION_SHOW_FLAG = false;
volatile boolean VOLUME_SHOW_FLAG = false;
volatile boolean TIME_SHOW_FLAG = true;

volatile int DIGITS [5];

volatile int Hour = 12;
volatile int Minute = 0;
volatile boolean TIME_FLAG = true;


//-----------------------------------------------------------------
void setup ()
{
  pinMode(MP1093_VOL_A, INPUT);
  pinMode(MP1093_VOL_B, INPUT);
  pinMode(MP1093_TUNE_A, INPUT);
  pinMode(MP1093_TUNE_B, INPUT);
  
  pinMode(KEY_0, INPUT);
  pinMode(KEY_1, INPUT);
  pinMode(KEY_2, INPUT);
  
  PCintPort::attachInterrupt(MP1093_VOL_A, &MP1093_VOL_A_ISR, RISING);
  PCintPort::attachInterrupt(MP1093_VOL_B, &MP1093_VOL_B_ISR, RISING);
  PCintPort::attachInterrupt(MP1093_TUNE_A, &MP1093_TUNE_A_ISR, RISING);
  PCintPort::attachInterrupt(MP1093_TUNE_B, &MP1093_TUNE_B_ISR, RISING);
  
  // инициализация модуля MP1091
  MP1091::Config (2);
  MP1091::AnodConfig (MP1091_DIG1_PIN, MP1091_DIG2_PIN,
                      MP1091_DIG3_PIN, MP1091_DIG4_PIN);
  MP1091::SegConfig (MP1091_A_MP1092_PIN, MP1091_B_MP1092_PIN,
                     MP1091_C_MP1092_PIN, MP1091_D_MP1092_PIN,
                     MP1091_E_MP1092_PIN, MP1091_F_MP1092_PIN,
                     MP1091_G_MP1092_PIN, MP1091_DP_MP1092_PIN,
                     MP1091_PP_MP1092_PIN);
  
  
  STATION_0 = 10 * (long (ReadWord (STATION_0_EEPROM_ADDRESS)));
  STATION_1 = 10 * (long (ReadWord (STATION_1_EEPROM_ADDRESS)));
  STATION_2 = 10 * (long (ReadWord (STATION_2_EEPROM_ADDRESS)));
  
  // инициализация модуля MP1090S
  MP1090S::InitI2C (MP1090S_RST_PIN);
  // инициализация канала ШИМ_0 модуля MP1092
  // для отображения двоеточия во время показа времени
  MP1092::SetPeriod (MP1092_ADDRESS, 0, 1000);
  MP1092::SetDutyCycle (MP1092_ADDRESS, 0, 50.0);
  MP1091::PPConfig (MP1092_ADDRESS, MP1092_PIN_PWM0);
  
  TWBR = 12;
  
  Serial.begin (9600);
  Serial.print ("EK004A + Arduino/DFRduino Nano");
  Serial.print ("\n");
  Serial.print ("Software: v1.1, 08.10.2013");
  Serial.print ("\n");
  Serial.print ("www.masterkit.ru");
  Serial.print ("\n");
  Serial.print ("\n");
    
  Serial.print ("time settings");
  Serial.print ("\n");
  // установка времени
  while (digitalRead(KEY_0))
  {
    SetDigits (Hour*100+Minute);
    MP1091_Show_Digits (DIGITS, 2, 5);
  }
  setTime (Hour, Minute, 0, 0, 0, 0);
  while(timeStatus() == timeNotSet);
  
  TIME_FLAG = false;
  
  for (int i=0; i<100; i++)
  {
    SetDigits ((hour()*100) + minute());
    MP1091_Show_Digits (DIGITS, 1, 5);
  }
  
  MP1091::PPConfig (MP1092_ADDRESS, MP1092_PIN_HIGH);
  MP1091_ClearDisplay ();
  TIME_SHOW_FLAG = false;

  PCintPort::attachInterrupt(KEY_0, &KEY_0_ISR, FALLING);
  PCintPort::attachInterrupt(KEY_1, &KEY_1_ISR, FALLING);
  PCintPort::attachInterrupt(KEY_2, &KEY_2_ISR, FALLING);
  
  // установка начальной громкости
  MP1090S::SetVolume (InitVolume);
  // установка начальной радиостанции
  // Если записана, устанавливается из памяти станция,
  // сохраненная по нажатию первой кнопки (KEY_0).
  // Если не записана, устанавливается станция по
  // умолчанию (InitStation)
  if ((STATION_0 >= 76000) && (STATION_0 <= 108000))
    MP1090S::SetStation (STATION_0);

  else
    MP1090S::SetStation (InitStation);
  
  SetDigits (MP1090S::GetStation () / 100);
  STATION_SHOW_FLAG = true;
}

//-----------------------------------------------------------------
void loop ()
{
  static int NoActivityCounter = 0;
  static int PrevSec;
  static int CurSec;
  static boolean MP1091_ON = true;
  static int ShowTimeCounter = 0;
  static boolean PPConfig_FLAG = false;

  // изменение громкости
  if ((VOLUME_FLAG == MP1093_ClockWiseRot) || (VOLUME_FLAG == MP1093_CounterClockWiseRot))
  {
    Serial.print ("Volume change");
    Serial.print ("\n");
    
    ChangeVolume ();
    SetDigits (MP1090S::GetVolume ());
    VOLUME_FLAG = MP1093_Unchanged;
    VOLUME_SHOW_FLAG = true;
    STATION_SHOW_FLAG = false;
    TIME_SHOW_FLAG = false;
    MP1091_ON = true;
    
    NoActivityCounter = 0;
        
    if (PPConfig_FLAG)
    {
      MP1091::PPConfig (MP1092_ADDRESS, MP1092_PIN_HIGH);
      PPConfig_FLAG = false;
    }
  }
  
  // смена радиостанции
  if ((TUNE_FLAG == MP1093_ClockWiseRot) || (TUNE_FLAG == MP1093_CounterClockWiseRot))
  {
    Serial.print ("Station change");
    Serial.print ("\n");
    
    ChangeChannel ();
    SetDigits (MP1090S::GetStation () / 100);
    TUNE_FLAG = MP1093_Unchanged;
    STATION_SHOW_FLAG = true;
    VOLUME_SHOW_FLAG = false;
    TIME_SHOW_FLAG = false;
    MP1091_ON = true;
    
    NoActivityCounter = 0;
    
    if (PPConfig_FLAG)
    {
      MP1091::PPConfig (MP1092_ADDRESS, MP1092_PIN_HIGH);
      PPConfig_FLAG = false;
    }
  }
  
  // обработка нажатия кнопок KEY_0, KEY_1, KEY_2
  // долгое нажатие (более 3 секунд): запоминание станции
  // короткое нажатие: воспроизведение сохраненной станции
  if (KEY_0_FLAG == KEY_PRESSED)
  {
    Serial.print ("KEY_0 pressed:");
    
    SetDigits (MP1090S::GetStation () / 100);
    
    STATION_SHOW_FLAG = true;
    VOLUME_SHOW_FLAG = false;
    TIME_SHOW_FLAG = false;
    MP1091_ON = true;

    NoActivityCounter = 0;
    
    if (PPConfig_FLAG)
    {
      MP1091::PPConfig (MP1092_ADDRESS, MP1092_PIN_HIGH);
      PPConfig_FLAG = false;
    }
    
    PrevSec = (minute()*60 + second());
    while (!digitalRead (KEY_0))
    {
      MP1091_Show_Digits (DIGITS, 1, 5);
    }
    CurSec = (minute()*60 + second());
    
    if ((CurSec-PrevSec) > 2)
    {
      Serial.print ("Station remembered");
      Serial.print ("\n");
      
      STATION_0 = MP1090S::GetStation ();
      RememberStation (STATION_0_EEPROM_ADDRESS, STATION_0);
      MP1091_ClearDisplay ();
      delay (1000);
    }
        
    else 
    {
      if ((STATION_0 >= 76000) || (STATION_0 <= 108000))
      {
        Serial.print ("Station set");
        Serial.print ("\n");
        
        MP1090S::SetStation (STATION_0);
        SetDigits (MP1090S::GetStation () / 100);
      }
    }
    
    KEY_0_FLAG = KEY_UNPRESSED;
  }
  
  
  if (KEY_1_FLAG == KEY_PRESSED)
  {
    Serial.print ("KEY_1 pressed:");
    
    SetDigits (MP1090S::GetStation () / 100);
    
    STATION_SHOW_FLAG = true;
    VOLUME_SHOW_FLAG = false;
    TIME_SHOW_FLAG = false;
    MP1091_ON = true;

    NoActivityCounter = 0;
    
    if (PPConfig_FLAG)
    {
      MP1091::PPConfig (MP1092_ADDRESS, MP1092_PIN_HIGH);
      PPConfig_FLAG = false;
    }
    
    PrevSec = (minute()*60 + second());
    while (!digitalRead (KEY_1))
    {
      MP1091_Show_Digits (DIGITS, 1, 5);
    }
    CurSec = (minute()*60 + second());
    
    if ((CurSec-PrevSec) > 2)
    {
      Serial.print ("Station remembered");
      Serial.print ("\n");
      
      STATION_1 = MP1090S::GetStation ();
      RememberStation (STATION_1_EEPROM_ADDRESS, STATION_1);
      MP1091_ClearDisplay ();
      delay (1000);
    }
        
    else 
    {
      if ((STATION_1 >= 76000) || (STATION_1 <= 108000))
      {
        Serial.print ("Station set");
        Serial.print ("\n");
        
        MP1090S::SetStation (STATION_1);
        SetDigits (MP1090S::GetStation () / 100);
      }
    }
    
    KEY_1_FLAG = KEY_UNPRESSED;
  }
  
  
  if (KEY_2_FLAG == KEY_PRESSED)
  {
    Serial.print ("KEY_2 pressed:");
    
    SetDigits (MP1090S::GetStation () / 100);
    
    STATION_SHOW_FLAG = true;
    VOLUME_SHOW_FLAG = false;
    TIME_SHOW_FLAG = false;
    MP1091_ON = true;
        
    NoActivityCounter = 0;
    
    if (PPConfig_FLAG)
    {
      MP1091::PPConfig (MP1092_ADDRESS, MP1092_PIN_HIGH);
      PPConfig_FLAG = false;
    }
    
    PrevSec = (minute()*60 + second());
    while (!digitalRead (KEY_2))
    {
      MP1091_Show_Digits (DIGITS, 1, 5);
    }
    CurSec = (minute()*60 + second());
    
    if ((CurSec-PrevSec) > 2)
    {
      Serial.print ("Station remembered");
      Serial.print ("\n");
      
      STATION_2 = MP1090S::GetStation ();
      RememberStation (STATION_2_EEPROM_ADDRESS, STATION_2);
      MP1091_ClearDisplay ();
      delay (1000);
    }
        
    else 
    {
      if ((STATION_2 >= 76000) || (STATION_2 <= 108000))
      {
        Serial.print ("Station set");
        Serial.print ("\n");
        
        MP1090S::SetStation (STATION_2);
        SetDigits (MP1090S::GetStation () / 100);
      }
    }
    
    KEY_2_FLAG = KEY_UNPRESSED;
  }
  
  // при отсутствии действий пользователя
  // периодическое отображение времени
  if (NoActivityCounter > 300)
  {
    if (NoActivityCounter == 301)
      ShowTimeCounter = 0;
    
    if (ShowTimeCounter <= 200)
    {  
      if (!PPConfig_FLAG)
      {
        Serial.print ("MP1091 ON");
        Serial.print ("\n");
        
        MP1091::PPConfig (MP1092_ADDRESS, MP1092_PIN_PWM0);
        PPConfig_FLAG = true;
        MP1091_ON = true;
        TIME_SHOW_FLAG = true;
        STATION_SHOW_FLAG = false;
        VOLUME_SHOW_FLAG = false;
      }
      SetDigits ((hour()*100) + minute());
    }
  
    else if ((ShowTimeCounter > 200) && (ShowTimeCounter <= 1000))
    {
      Serial.print ("MP1091 OFF");
      Serial.print ("\n");
              
      if (PPConfig_FLAG)
      {
        MP1091::PPConfig (MP1092_ADDRESS, MP1092_PIN_HIGH);
        MP1091_ClearDisplay ();
        PPConfig_FLAG = false;
        MP1091_ON = false;     
      }
    }
      
    else if (ShowTimeCounter == 1001)
      {
        ShowTimeCounter = 0;
      }
      
    ShowTimeCounter++;
  }

  if (MP1091_ON)
    MP1091_Show_Digits (DIGITS, 1, 5);
    
  NoActivityCounter++;
  
  if (NoActivityCounter == 0xFFFF)
    NoActivityCounter = 301;
}

//-----------------------------------------------------------------
// обработка прерывания от MP1093 (громкость)
// при вращении по часовой стрелке
void MP1093_VOL_A_ISR ()
{
  if (PCintPort::arduinoPin == MP1093_VOL_A)
  {
   delay (100);
   
   if ((digitalRead (MP1093_VOL_A)) && (!digitalRead (MP1093_VOL_B)))
   {
     if (TIME_FLAG)
    {
      if (Minute<59)
        Minute++;
      
      else
        Minute = 0;
    }
     
     else
       VOLUME_FLAG = MP1093_ClockWiseRot;
   }
  }
}
//-----------------------------------------------------------------
// обработка прерывания от MP1093 (громкость)
// при вращении против часовой стрелки
void MP1093_VOL_B_ISR ()
{
  if (PCintPort::arduinoPin == MP1093_VOL_B)
  {
    delay (100);
    if ((digitalRead (MP1093_VOL_B)) && (!digitalRead (MP1093_VOL_A)))
    {
      if (TIME_FLAG)
    {
      if (Minute>0)
        Minute--;
      
      else
        Minute = 59;
    }
      
     else
       VOLUME_FLAG = MP1093_CounterClockWiseRot;
    }
  }
}
//-----------------------------------------------------------------
// обработка прерывания от MP1093 (настройка)
// при вращении по часовой стрелке
void MP1093_TUNE_A_ISR ()
{
  if (PCintPort::arduinoPin == MP1093_TUNE_A)
  {
     delay (100);
     if ((digitalRead (MP1093_TUNE_A)) && (!digitalRead (MP1093_TUNE_B)))
     {
        if (TIME_FLAG)
        {
           if (Hour < 23)
              Hour++;
           else
              Hour = 0;
        }
        else
          TUNE_FLAG = MP1093_ClockWiseRot;
     }
  }
}
//-----------------------------------------------------------------
// обработка прерывания от MP1093 (настройка)
// при вращении против часовой стрелки
void MP1093_TUNE_B_ISR ()
{
  if (PCintPort::arduinoPin == MP1093_TUNE_B)
  {
   delay (100);
   if ((digitalRead (MP1093_TUNE_B)) && (!digitalRead (MP1093_TUNE_A)))
   {
     if (TIME_FLAG)
    {
      if (Hour>0)
        Hour--;
      
      else
        Hour = 23;
    }
     
     else
       TUNE_FLAG = MP1093_CounterClockWiseRot;
   }
  }
}
//-----------------------------------------------------------------
// обработка прерывания при нажатии кнопки KEY_0
void KEY_0_ISR ()
{
  if (PCintPort::arduinoPin == KEY_0)
  {
   delay (100);
   if (!digitalRead (KEY_0))
   {
      KEY_0_FLAG = KEY_PRESSED;
   }
  }
}
//-----------------------------------------------------------------
// обработка прерывания при нажатии кнопки KEY_1
void KEY_1_ISR ()
{
  if (PCintPort::arduinoPin == KEY_1)
  {
   delay (100);
   
     if (!digitalRead (KEY_1))
     {
       KEY_1_FLAG = KEY_PRESSED;
     }
  }
}
//-----------------------------------------------------------------
// обработка прерывания при нажатии кнопки KEY_2
void KEY_2_ISR ()
{
  if (PCintPort::arduinoPin == KEY_2)
  {
   delay (100);
   
   if (!digitalRead (KEY_2))
     {
       KEY_2_FLAG = KEY_PRESSED;
     }
  }
}
//-----------------------------------------------------------------
// разбиение числа на цифры для отображения на MP1091
void SetDigits (long number)
{
  byte Digit3;
  byte Digit2;
  byte Digit1;
  byte Digit0;
  
  long InputData = number;

  if (InputData < 10)
  {
    Digit3 = byte (number);
    Digit0 = 10;
    Digit1 = 10;
    Digit2 = 10;
    
    DIGITS [4] = 5;
  }
  
  else if ((InputData < 100) && (InputData >=10))
    {
      Digit3 = byte (number%10);
      number=number/10;
      Digit2 = byte (number); 
      Digit0 = 10;
      Digit1 = 10;
      
      DIGITS [4] = 5;
    }
  
  else if ((InputData >= 100) && (InputData <1000))
  {
    Digit3 = byte (number%10);
    number=number/10;
    Digit2 = byte (number%10);
    number=number/10;
    Digit1 = byte (number%10);
    Digit0 = 10;
    
    DIGITS [4] = 5;
  }
  
  else if ((InputData >= 1000) && (InputData <10000))
  {
    Digit3 = byte (number%10);
    number=number/10;
    Digit2 = byte (number%10);
    number=number/10;
    Digit1 = byte (number%10);
    number=number/10;
    Digit0 = byte (number);
    
    DIGITS [4] = 5;
  }
  
  DIGITS [0] = Digit0;
  DIGITS [1] = Digit1;
  DIGITS [2] = Digit2;
  DIGITS [3] = Digit3;
}
//--------------------------------------------------------------
void MP1091_ActivateDigit (int Num)
{
  int Anods [4] = {MP1091_DIG1_PIN, MP1091_DIG2_PIN,
                   MP1091_DIG3_PIN, MP1091_DIG4_PIN};
  
  for (int i=0; i<4; i++)
    digitalWrite(Anods [i], HIGH);
    
  digitalWrite(Anods [Num], LOW);
  
}
//--------------------------------------------------------------
// отображение числа на MP1091
void MP1091_Show_Digits (volatile int* Digits, int mode, int pause)
{
  boolean Point;
  
  void (*DigFunc[11])(boolean, int) = {LED_DIGIT_0, LED_DIGIT_1, LED_DIGIT_2,
                               LED_DIGIT_3, LED_DIGIT_4, LED_DIGIT_5,
                               LED_DIGIT_6, LED_DIGIT_7, LED_DIGIT_8,
                               LED_DIGIT_9, LED_DIGIT_NO_Digit};
  
  if (STATION_SHOW_FLAG)
    DIGITS [4] = 2;
  
  else if (TIME_SHOW_FLAG)
  {
    if (DIGITS[1] == 10)
      DIGITS[1] = 0;
    
    if (DIGITS[2] == 10)
      DIGITS[2] = 0;    
  }
  
  for (int i=0; i<4; i++)
  {
    if ((i) == *(Digits+4))
      Point = true;
    else
      Point = false;
      
    (*DigFunc[*(Digits+i)])(Point, mode);
    MP1091_ActivateDigit (i);
    delay (pause);
  }
}
//--------------------------------------------------------------
void MP1091_ClearDisplay ()
{
  int Anods [4] = {MP1091_DIG1_PIN, MP1091_DIG2_PIN,
                   MP1091_DIG3_PIN, MP1091_DIG4_PIN};
  
  for (int i=0; i<4; i++)
    digitalWrite(Anods [i], HIGH);
}
//-----------------------------------------------------------------
// изменение громкости
void ChangeVolume ()
{
  byte PrevVolume = MP1090S::GetVolume ();
  byte NewVolume = PrevVolume;
  
  if (VOLUME_FLAG == MP1093_ClockWiseRot)
  {
    if (NewVolume < MAX_VOLUME)
      NewVolume++;
  }
  
  else if (VOLUME_FLAG == MP1093_CounterClockWiseRot)
  {
    if (NewVolume > MIN_VOLUME)
      NewVolume--;
  }
  
  if (NewVolume != PrevVolume)
    MP1090S::SetVolume (NewVolume);
}
//-----------------------------------------------------------------
// смена радиостанции
void ChangeChannel ()
{
  byte StationSpace = MP1090S::GetSpace ();
  byte Band = MP1090S::GetBand ();
  long PrevStation = MP1090S::GetStation ();
  long NewStation = PrevStation;
  
  
  if (TUNE_FLAG == MP1093_ClockWiseRot)
  {
    if ((Band == MHz87_5_108) || (Band == MHz76_108))
    {
      if ((NewStation + StationSpace) <= 108000)
        NewStation += StationSpace;
    }
    
    else if (Band == MHz76_90)
    {
      if ((NewStation + StationSpace) <= 90000)
        NewStation += StationSpace;
    }
  }

  else if (TUNE_FLAG == MP1093_CounterClockWiseRot)
  {
    if ((Band == MHz76_90) || (Band == MHz76_108))
    {
      if ((NewStation - StationSpace) >= 76000)
        NewStation -= StationSpace;
    }
    
    else if (Band == MHz87_5_108)
    {
      if ((NewStation - StationSpace) >= 87500)
        NewStation -= StationSpace;
    }
  }
  
  if (NewStation != PrevStation)
    MP1090S::SetStation (NewStation);
}
//-----------------------------------------------------------------
// запись радиостанции в память микроконтроллера
void RememberStation (int StartAddress, long StationToRemember)
{
  word Station;
  Station = word (StationToRemember / 10);
  
  EEPROM.write(StartAddress, highByte (Station));
  EEPROM.write(StartAddress+1, lowByte (Station));
}
//-----------------------------------------------------------------
// чтение двух байт (радиостанции) из памяти микроконтроллера
word ReadWord (int StartAddress)
{
  return (word (EEPROM.read(StartAddress), EEPROM.read(StartAddress+1)));
}
//-----------------------------------------------------------------
void LED_DIGIT_1 (boolean Point, int mode)
{
  int PointLED;

  if (Point)
    PointLED = (mode<<(3*2));
  else
    PointLED = LED_OFF(3);
  
  Wire.beginTransmission(MP1092_ADDRESS);
  Wire.write (MP1092_LS0 | MP1092_AUTO_INCR);
  Wire.write (LED_OFF(0)|(mode<<(1*2))|(mode<<(2*2))|LED_OFF(3));
  Wire.write (LED_OFF(0)|LED_OFF(1)|LED_OFF(2)|PointLED);
  Wire.endTransmission();
}
//---------------------------------------------------------------
void LED_DIGIT_2 (boolean Point, int mode)
{
  int PointLED;

  if (Point)
    PointLED = (mode<<(3*2));
  else
    PointLED = LED_OFF(3);
  
  Wire.beginTransmission(MP1092_ADDRESS);
  Wire.write (MP1092_LS0 | MP1092_AUTO_INCR);
  Wire.write ((mode<<(0*2))|(mode<<(1*2))|LED_OFF(2)|(mode<<(3*2)));
  Wire.write ((mode<<(0*2))|LED_OFF(1)|(mode<<(2*2))|PointLED);
  Wire.endTransmission();
}
//---------------------------------------------------------------
void LED_DIGIT_3 (boolean Point, int mode)
{
  int PointLED;

  if (Point)
    PointLED = (mode<<(3*2));
  else
    PointLED = LED_OFF(3);
  
  Wire.beginTransmission(MP1092_ADDRESS);
  Wire.write (MP1092_LS0 | MP1092_AUTO_INCR);
  Wire.write ((mode<<(0*2))|(mode<<(1*2))|(mode<<(2*2))|(mode<<(3*2)));
  Wire.write (LED_OFF(0)|LED_OFF(1)|(mode<<(2*2))|PointLED);
  Wire.endTransmission();
}
//---------------------------------------------------------------
void LED_DIGIT_4 (boolean Point, int mode)
{
  int PointLED;

  if (Point)
    PointLED = (mode<<(3*2));
  else
    PointLED = LED_OFF(3);
  
  Wire.beginTransmission(MP1092_ADDRESS);
  Wire.write (MP1092_LS0 | MP1092_AUTO_INCR);
  Wire.write (LED_OFF(0)|(mode<<(1*2))|(mode<<(2*2))|LED_OFF(3));
  Wire.write (LED_OFF(0)|(mode<<(1*2))|(mode<<(2*2))|PointLED);
  Wire.endTransmission();
}
//---------------------------------------------------------------
void LED_DIGIT_5 (boolean Point, int mode)
{
  int PointLED;

  if (Point)
    PointLED = (mode<<(3*2));
  else
    PointLED = LED_OFF(3);
  
  Wire.beginTransmission(MP1092_ADDRESS);
  Wire.write (MP1092_LS0 | MP1092_AUTO_INCR);
  Wire.write ((mode<<(0*2))|LED_OFF(1)|(mode<<(2*2))|(mode<<(3*2)));
  Wire.write (LED_OFF(0)|(mode<<(1*2))|(mode<<(2*2))|PointLED);
  Wire.endTransmission();
}
//---------------------------------------------------------------
void LED_DIGIT_6 (boolean Point, int mode)
{
  int PointLED;

  if (Point)
    PointLED = (mode<<(3*2));
  else
    PointLED = LED_OFF(3);
  
  Wire.beginTransmission(MP1092_ADDRESS);
  Wire.write (MP1092_LS0 | MP1092_AUTO_INCR);
  Wire.write ((mode<<(0*2))|LED_OFF(1)|(mode<<(2*2))|(mode<<(3*2)));
  Wire.write ((mode<<(0*2))|(mode<<(1*2))|(mode<<(2*2))|PointLED);
  Wire.endTransmission();
}
//---------------------------------------------------------------
void LED_DIGIT_7 (boolean Point, int mode)
{
  int PointLED;

  if (Point)
    PointLED = (mode<<(3*2));
  else
    PointLED = LED_OFF(3);
  
  Wire.beginTransmission(MP1092_ADDRESS);
  Wire.write (MP1092_LS0 | MP1092_AUTO_INCR);
  Wire.write ((mode<<(0*2))|(mode<<(1*2))|(mode<<(2*2))|LED_OFF(3));
  Wire.write (LED_OFF(0)|LED_OFF(1)|LED_OFF(2)|PointLED);
  Wire.endTransmission();
}
//---------------------------------------------------------------
void LED_DIGIT_8 (boolean Point, int mode)
{
  int PointLED;

  if (Point)
    PointLED = (mode<<(3*2));
  else
    PointLED = LED_OFF(3);
  
  Wire.beginTransmission(MP1092_ADDRESS);
  Wire.write (MP1092_LS0 | MP1092_AUTO_INCR);
  Wire.write ((mode<<(0*2))|(mode<<(1*2))|(mode<<(2*2))|(mode<<(3*2)));
  Wire.write ((mode<<(0*2))|(mode<<(1*2))|(mode<<(2*2))|PointLED);
  Wire.endTransmission();
}
//---------------------------------------------------------------
void LED_DIGIT_9 (boolean Point, int mode)
{
  int PointLED;

  if (Point)
    PointLED = (mode<<(3*2));
  else
    PointLED = LED_OFF(3);
  
  Wire.beginTransmission(MP1092_ADDRESS);
  Wire.write (MP1092_LS0 | MP1092_AUTO_INCR);
  Wire.write ((mode<<(0*2))|(mode<<(1*2))|(mode<<(2*2))|(mode<<(3*2)));
  Wire.write (LED_OFF(0)|(mode<<(1*2))|(mode<<(2*2))|PointLED);
  Wire.endTransmission();
}
//---------------------------------------------------------------
void LED_DIGIT_0 (boolean Point, int mode)
{
  int PointLED;

  if (Point)
    PointLED = (mode<<(3*2));
  else
    PointLED = LED_OFF(3);
  
  Wire.beginTransmission(MP1092_ADDRESS);
  Wire.write (MP1092_LS0 | MP1092_AUTO_INCR);
  Wire.write ((mode<<(0*2))|(mode<<(1*2))|(mode<<(2*2))|(mode<<(3*2)));
  Wire.write ((mode<<(0*2))|(mode<<(1*2))|LED_OFF(2)|PointLED);
  Wire.endTransmission();
}
//--------------------------------------------------------------
void LED_DIGIT_NO_Digit (boolean Point, int mode)
{
  int PointLED;

  if (Point)
    PointLED = (mode<<(3*2));
  else
    PointLED = LED_OFF(3);
  
  Wire.beginTransmission(MP1092_ADDRESS);
  Wire.write (MP1092_LS0 | MP1092_AUTO_INCR);
  Wire.write (LED_OFF(0)|LED_OFF(1)|LED_OFF(2)|LED_OFF(3));
  Wire.write (LED_OFF(0)|LED_OFF(1)|LED_OFF(2)|PointLED);
  Wire.endTransmission();
}

